/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RSKRX111 Starter-Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

/****** Addresses of sfrs and peripheral identifiers ****************/
#define P1_BASE_ADDR       (0x0008C001u)
#define P5_BASE_ADDR       (0x0008C005u)
#define PA_BASE_ADDR       (0x0008C00Au)
#define PB_BASE_ADDR       (0x0008C00Bu)

/****** SFRs and identifiers used for LED demo **********************/
#define P1_DDR             (*(volatile EVENACCESS unsigned char*) (P1_BASE_ADDR + 0x00u))
#define P1_DR              (*(volatile EVENACCESS unsigned char*) (P1_BASE_ADDR + 0x20u))
#define P5_DDR             (*(volatile EVENACCESS unsigned char*) (P5_BASE_ADDR + 0x00u))
#define P5_DR              (*(volatile EVENACCESS unsigned char*) (P5_BASE_ADDR + 0x20u))
#define PA_DDR             (*(volatile EVENACCESS unsigned char*) (PA_BASE_ADDR + 0x00u))
#define PA_DR              (*(volatile EVENACCESS unsigned char*) (PA_BASE_ADDR + 0x20u))
#define PB_DDR             (*(volatile EVENACCESS unsigned char*) (PB_BASE_ADDR + 0x00u))
#define PB_DR              (*(volatile EVENACCESS unsigned char*) (PB_BASE_ADDR + 0x20u))

/****** Assign LEDs to Ports ****************************************/
#define LED0_BIT           (7u)      // LED 0 on RSKRX111 => PB7
#define LED1_BIT           (0u)      // LED 1 on RSKRX111 => PA0
#define LED2_BIT           (4u)      // LED 2 on RSKRX111 => P54
#define LED3_BIT           (7u)      // LED 2 on RSKRX111 => P17

#define LED0_MASK          (1u << LED0_BIT)
#define LED1_MASK          (1u << LED1_BIT)
#define LED2_MASK          (1u << LED2_BIT)
#define LED3_MASK          (1u << LED3_BIT)

#define LED0_PORT_DATA      PB_DR
#define LED0_PORT_DIRECTION PB_DDR
#define LED1_PORT_DATA      PA_DR
#define LED1_PORT_DIRECTION PA_DDR
#define LED2_PORT_DATA      P5_DR
#define LED2_PORT_DIRECTION P5_DDR
#define LED3_PORT_DATA      P1_DR
#define LED3_PORT_DIRECTION P1_DDR

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LED0_PORT_DATA      |= LED0_MASK;  // Switch off LED0 initially (active low)
  LED0_PORT_DIRECTION |= LED0_MASK;  // Switch LED0-Port to output
  LED1_PORT_DATA      |= LED1_MASK;  // Switch off LED1 initially (active low)
  LED1_PORT_DIRECTION |= LED1_MASK;  // Switch LED1-Port to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    LED0_PORT_DATA &= ~(LED0_MASK);  // Switch LED on, active low
  } else if (Index == 1) {
    LED1_PORT_DATA &= ~(LED1_MASK);  // Switch LED on, active low
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    LED0_PORT_DATA |= (LED0_MASK);   // Switch LED off, set port high
  } else if (Index == 1) {
    LED1_PORT_DATA |= (LED1_MASK);   // Switch LED off, set port high
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    LED0_PORT_DATA ^= LED0_MASK;
  } else if (Index == 1) {
    LED1_PORT_DATA ^= LED1_MASK;
  }
}

/****** End Of File  ************************************************/
