/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RSKRX113 Starter-Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

#define P2_BASE_ADDR       (0x0008C002u)
#define P2_DDR             (*(volatile EVENACCESS unsigned char*) (P2_BASE_ADDR + 0x00u))
#define P2_DR              (*(volatile EVENACCESS unsigned char*) (P2_BASE_ADDR + 0x20u))
#define LED0_BIT           (2u)  // LED 0 on RSKRX113 => P22
#define LED1_BIT           (3u)  // LED 1 on RSKRX113 => P23

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  P2_DR  |= (1u << LED0_BIT) | (1u << LED1_BIT);  // Switch off LEDs initially (active low)
  P2_DDR |= (1u << LED0_BIT) | (1u << LED1_BIT);  // Switch LEDs to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    P2_DR &= ~(1u << LED0_BIT);  // Switch LED on, active low
  } else if (Index == 1) {
    P2_DR &= ~(1u << LED1_BIT);  // Switch LED on, active low
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    P2_DR |= (1u << LED0_BIT);   // Switch LED off, set port high
  } else if (Index == 1) {
    P2_DR |= (1u << LED1_BIT);   // Switch LED off, set port high
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    P2_DR ^= (1u << LED0_BIT);
  } else if (Index == 1) {
    P2_DR ^= (1u << LED1_BIT);
  }
}

/****** End Of File *************************************************/
