/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_RX210.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (50000000u / 2u / 8u)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKRX.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
//
// Protect register
//
#define PRCR                 (*(volatile EVENACCESS OS_U16*)(0x000803FEu))

//
// SYSTEM module
//
#define SYSTEM_BASE_ADDR     (0x00080000u)
#define MSTPCRA              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x010u))  // Modul stop register A
#define MSTPCRB              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x014u))  // Modul stop register B
#define MSTPCRC              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x018u))  // Modul stop register C
#define SCKCR                (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x020u))  // System clock control register
#define SCKCR3               (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x026u))  // System clock control register 3
#define PLLCR                (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x028u))  // PLL control register
#define PLLCR2               (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x02Au))  // PLL control register 2
#define BCKCR                (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x030u))  // External bus clock control register
#define MOSCCR               (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x032u))  // Main clock oscillator control register
#define MOSCWTCR             (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x0A2u))  // Main clock oscillator wait control register
#define PLLWTCR              (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x0A6u))  // PLL wait control register

//
// Assign timer used for OS
//
#define CMT0_BASE_ADDR       (0x00088000u)
#define CMT0_MSTPCR          (MSTPCRA)
#define CMT0_MSTPCR_BIT      (15u)
#define CMT0_VECT            28u
#define CMT0_IPR             (IPR04)
#define CMT0_IER             (IER03)
#define CMT0_IER_BIT         (4u)
#define OS_TIMER_INT_PRIO    (0x01u)     // Lowest priority
#define OS_TIMER_VECT        CMT0_VECT
#define CMT0_CMSTR0          (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x00u))
#define CMT0_CMCR            (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x02u))
#define CMT0_CMCNT           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x04u))
#define CMT0_CMCOR           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x06u))

//
// Interrupt controller
//
#define IRR_BASE_ADDR        (0x00087000u)
#define ISEL_BASE_ADDR       (0x00087100u)

#define IER03                (*(volatile EVENACCESS OS_U8*) (0x00087203u))
#define IPR04                (*(volatile EVENACCESS OS_U8*) (0x00087304u))

#define IRR_PEND             (*(volatile EVENACCESS OS_U8*) (IRR_BASE_ADDR + OS_TIMER_VECT))
#define ISEL_SRC             (*(volatile EVENACCESS OS_U8*) (ISEL_BASE_ADDR + OS_TIMER_VECT))

/*********************************************************************
*
*       Const data
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration of Option Bytes and ID Code
*
*       We set all ID Code bytes explicitly to 0xFF which is the default value.
*       IAR handles these bytes in IAR linker files.
*       RX GNU handles these bytes in vects.c and GNU linker script.
*/
#ifdef __RX
  #pragma address _MDEreg=0xFFFFFF80            // MDE register (Single Chip Mode)
  #ifdef __BIG
    const unsigned long _MDEreg = 0xFFFFFFF8;   // Big endian
  #else
    const unsigned long _MDEreg = 0xFFFFFFFF;   // Little endian
  #endif
  #pragma address _Dummy=0xFFFFFF84             // Dummy value
  const unsigned long _Dummy  = 0xFFFFFFFF;
  #pragma address _OFS1=0xFFFFFF88              // Option function select register 1 (OFS1)
  const unsigned long _OFS1   = 0xFFFFFFFF;
  #pragma address _OFS2=0xFFFFFF8C              // Option function select register 2 (OFS2)
  const unsigned long _OFS2   = 0xFFFFFFFF;
  #pragma address _IDCode0=0xFFFFFFA0           // ID Code 0
  const unsigned long _IDCode0  = 0xFFFFFFFF;
  #pragma address _IDCode1=0xFFFFFFA4           // ID Code 1
  const unsigned long _IDCode1  = 0xFFFFFFFF;
  #pragma address _IDCode2=0xFFFFFFA8           // ID Code 2
  const unsigned long _IDCode2  = 0xFFFFFFFF;
  #pragma address _IDCode3=0xFFFFFFAC           // ID Code 3
  const unsigned long _IDCode3  = 0xFFFFFFFF;
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return CMT0_CMCNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return IRR_PEND;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#ifdef __ICCRX__
  #pragma vector = (OS_TIMER_VECT)
  static __interrupt void SysTick_Handler(void) {
#endif
#ifdef __RX
  #pragma interrupt (SysTick_Handler(vect=OS_TIMER_VECT))
  static void SysTick_Handler(void) {
#endif
#ifdef __GNUC__
  void __attribute__ ((interrupt)) Excep_CMT0_CMI0(void) {
#endif
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_TickCnt++;
#endif
  OS_INT_EnterNestable();
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_Process();
#endif
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  PRCR         = 0xA50Bu;                                // Protect off
  CMT0_MSTPCR &= ~(1u << CMT0_MSTPCR_BIT);               // Activate CMT unit
  CMT0_CMSTR0  = 0x00u;                                  // Stop counter
  CMT0_CMCR    = (1u << 6u);                             // Compare match interrupt enable, PCLK/8
  CMT0_CMCOR   = (OS_U16)(OS_TIMER_FREQ / OS_INT_FREQ);  // Count value for 1 msec resolution
  //
  // Enable timer interrupt in interrupt controller
  //
  CMT0_IPR  = OS_TIMER_INT_PRIO;                         // Set interrupt priority, lowest priority
  CMT0_IER |= (1u << CMT0_IER_BIT);                      // Enable CMT interrupt
  IRR_PEND  = 0u;                                        // Clear interrupt request
  ISEL_SRC  = 0u;                                        // Select interrupt source CPU
  //
  // Start timer
  //
  CMT0_CMSTR0  = 0x01u;                                  // Start counter
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Configure and initialize SEGGER SystemView
  //
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_Conf();
#endif
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SetpfOnRx(OS_COM_OnRx);
  JLINKRX_SetpfOnTx(OS_COM_OnTx);
  JLINKRX_SetpfGetNextChar(OS_COM_GetNextChar);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  PRCR = 0xA500u;                                        // Protect on
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
#if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
  #ifdef __ICCRX__
    __wait_for_interrupt();
  #endif
  #ifdef __RX
    wait();
  #endif
  #ifdef __GNUC__
    __builtin_rx_wait();
  #endif
#endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       HardwareSetup()
*
*  Function description
*    Initialize clock generation and PLL
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before HardwareSetup() is called,
*    no access or call which relies on any variable can be used here
*    and no variables can be set to a specific value as long as those variables
*    are not declared as "no init".
*
*    This function is called from startup code.
*/
void HardwareSetup(void) {
  unsigned int i;
  //
  // Enable PLL
  // We assume a 20 MHz external crystal
  //
  PRCR     = 0xA503u;                   // Protect off
  MOSCWTCR = 0x0Du;                     // 131072 state, wait over 10ms
  PLLWTCR  = 0x0Eu;                     // 2097152 state, wait over 12ms  @PLL=100MHz(20MHz/2*10)
  PLLCR    = 0x0901u;                   // x10 /2 @PLL
  MOSCCR   = 0x02u;                     // External oscillation input selection
  PLLCR2   = 0x00u;                     // Enable PLL
  for (i = 0u ; i < 0x168u ; i++) {     // Wait over 12ms
  }
  SCKCR    = 0x21022222u;               // ICK=PLL/2,FCK,PCK,BCL=PLL/4
  SCKCR3   = 0x0400u;                   // LOCO -> PLL
  PRCR     = 0xA500u;                   // Protect on
}

#ifdef __ICCRX__
/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  HardwareSetup();
  return 1;  // Always return 1 to enable segment initilization !
}
#endif

/*************************** End of file ****************************/
