/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RSKRX24T eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

#define PB_BASE_ADDR       (0x0008C00Bu)
#define PB_DDR             (*(volatile EVENACCESS unsigned char*) (PB_BASE_ADDR + 0x00u))
#define PB_DR              (*(volatile EVENACCESS unsigned char*) (PB_BASE_ADDR + 0x20u))

#define PE_BASE_ADDR       (0x0008C00Eu)
#define PE_DDR             (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x00u))
#define PE_DR              (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x20u))

/****** Assign LEDs to Ports ****************************************/
#define LED0_BIT           (3u)      // LED 0 (green)  on RSKRX24T
#define LED1_BIT           (5u)      // LED 1 (orange) on RSKRX24T
#define LED0_MASK          (1u << LED0_BIT)
#define LED1_MASK          (1u << LED1_BIT)

#define LED_PORTB_DATA      PB_DR
#define LED_PORTE_DATA      PE_DR
#define LED_PORTB_DIRECTION PB_DDR
#define LED_PORTE_DIRECTION PE_DDR

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Initialize LED 0
  //
  LED_PORTB_DATA      |= LED0_MASK;  // Switch off led initially (active low)
  LED_PORTB_DIRECTION |= LED0_MASK;  // Switch LED-Port to output
  //
  // Initialize LED 1
  //
  LED_PORTE_DATA      |= LED1_MASK;  // Switch off led initially (active low)
  LED_PORTE_DIRECTION |= LED1_MASK;  // Switch LED-Port to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    LED_PORTB_DATA &= ~(LED0_MASK);  // Switch LED on, active low
  } else if (Index == 1) {
    LED_PORTE_DATA &= ~(LED1_MASK);  // Switch LED on, active low
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    LED_PORTB_DATA |= (LED0_MASK);  // Switch LED off, set port high
  } else if (Index == 1) {
    LED_PORTE_DATA |= (LED1_MASK);  // Switch LED off, set port high
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    LED_PORTB_DATA ^= LED0_MASK;
  } else if (Index == 1) {
    LED_PORTE_DATA ^= LED1_MASK;
  }
}

/****** End Of File *************************************************/
