/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RPBRX62N Promotion board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PA_BASE_ADDR       (0x0008C00A)

/****** SFRs and identifiers used for LED demo **********************/
#define PA_DDR             (*(volatile EVENACCESS unsigned char*) (PA_BASE_ADDR + 0x00))
#define PA_DR              (*(volatile EVENACCESS unsigned char*) (PA_BASE_ADDR + 0x20))

/****** Assign LEDs to Ports ****************************************/
#define LED0_BIT           (0u)              // LED 0 on RPBRX62N
#define LED1_BIT           (1u)              // LED 1 on RPBRX62N
#define LED2_BIT           (2u)              // LED 2 on RPBRX62N
#define LED0_MASK          (1u << LED0_BIT)
#define LED1_MASK          (1u << LED1_BIT)
#define LED2_MASK          (1u << LED2_BIT)
#define LED_MASK_ALL       (LED0_MASK | LED1_MASK | LED2_MASK)

#define LED_PORT_DATA      PA_DR
#define LED_PORT_DIRECTION PA_DDR

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LED_PORT_DATA      |= LED_MASK_ALL;  // Switch off leds initially (active low)
  LED_PORT_DIRECTION |= LED_MASK_ALL;  // Switch LED-Port to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 3) {
    LED_PORT_DATA &= ~(1u << Index);   // Switch LED on, active low
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 3) {
    LED_PORT_DATA |= (1u << Index);   // Switch LED off, set port high
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < 3) {
    LED_PORT_DATA ^= (1u << Index);   // Toggle LED
  }
}

/****** End Of File  ************************************************/
