/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_RX62N.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (96000000u / 2u / 8u)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKRX.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
//
// SYSTEM module
//
#define SYSTEM_BASE_ADDR     (0x00080000u)
#define SYSCR0               (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x006u))
#define MSTPCRA              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x010u))
#define MSTPCRB              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x014u))
#define MSTPCRC              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x018u))
#define SCKCR                (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x020u))

//
// IOPORT
//
#define IOPORT_PF4BUS        (*(volatile EVENACCESS OS_U8*) (0x0008C104u))
#define IOPORT_PF5BUS        (*(volatile EVENACCESS OS_U8*) (0x0008C105u))
#define IOPORT_PF6BUS        (*(volatile EVENACCESS OS_U8*) (0x0008C106u))

//
// Buses
//
#define BSC_SDCCR            (*(volatile EVENACCESS OS_U8*) (0x00083C00u))
#define BSC_SDCMOD           (*(volatile EVENACCESS OS_U8*) (0x00083C01u))
#define BSC_SDRFCR           (*(volatile EVENACCESS OS_U16*)(0x00083C14u))
#define BSC_SDRFEN           (*(volatile EVENACCESS OS_U8*) (0x00083C16u))
#define BSC_SDICR            (*(volatile EVENACCESS OS_U8*) (0x00083C20u))
#define BSC_SDIR             (*(volatile EVENACCESS OS_U16*)(0x00083C24u))
#define BSC_SDADR            (*(volatile EVENACCESS OS_U8*) (0x00083C40u))
#define BSC_SDTR             (*(volatile EVENACCESS OS_U32*)(0x00083C44u))
#define BSC_SDMOD            (*(volatile EVENACCESS OS_U32*)(0x00083C48u))
#define BSC_SDSR             (*(volatile EVENACCESS OS_U8*) (0x00083C50u))

#define INIRQ                0u
#define INIST                3u
#define BSIZE                4u
#define REFW                 12u
#define RFC                  0u
#define EXENB                0u

//
// Assign timer used for OS
//
#define CMT0_BASE_ADDR       (0x00088000u)
#define CMT0_MSTPCR          (MSTPCRA)
#define CMT0_MSTPCR_BIT      (15u)
#define CMT0_VECT            28u
#define CMT0_IPR             (IPR04)
#define CMT0_IER             (IER03)
#define CMT0_IER_BIT         (4u)
#define OS_TIMER_INT_PRIO    (0x01u)     // Lowest priority
#define OS_TIMER_VECT        CMT0_VECT
#define CMT0_CMSTR0          (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x00u))
#define CMT0_CMCR            (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x02u))
#define CMT0_CMCNT           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x04u))
#define CMT0_CMCOR           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x06u))

//
// Interrupt controller
//
#define IRR_BASE_ADDR        (0x00087000u)
#define ISEL_BASE_ADDR       (0x00087100u)

#define IER03                (*(volatile EVENACCESS OS_U8*) (0x00087203u))
#define IPR04                (*(volatile EVENACCESS OS_U8*) (0x00087304u))

#define IRR_PEND             (*(volatile EVENACCESS OS_U8*) (IRR_BASE_ADDR + OS_TIMER_VECT))
#define ISEL_SRC             (*(volatile EVENACCESS OS_U8*) (ISEL_BASE_ADDR + OS_TIMER_VECT))

/*********************************************************************
*
*       Const data
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration of Option Bytes and ID Code
*
*       We set all ID Code bytes explicitly to 0xFF which is the default value.
*       IAR handles these bytes in IAR linker files.
*       RX GNU handles these bytes in vects.c and GNU linker script.
*/
#ifdef __RX
  #pragma address _IDCode0=0xFFFFFFA0           // ID Code 0
  const unsigned long _IDCode0  = 0xFFFFFFFF;
  #pragma address _IDCode1=0xFFFFFFA4           // ID Code 1
  const unsigned long _IDCode1  = 0xFFFFFFFF;
  #pragma address _IDCode2=0xFFFFFFA8           // ID Code 2
  const unsigned long _IDCode2  = 0xFFFFFFFF;
  #pragma address _IDCode3=0xFFFFFFAC           // ID Code 3
  const unsigned long _IDCode3  = 0xFFFFFFFF;
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return CMT0_CMCNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return IRR_PEND;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#ifdef __ICCRX__
  #pragma vector = (OS_TIMER_VECT)
  static __interrupt void SysTick_Handler(void) {
#endif
#ifdef __RX
  #pragma interrupt (SysTick_Handler(vect=OS_TIMER_VECT))
  static void SysTick_Handler(void) {
#endif
#ifdef __GNUC__
  void __attribute__ ((interrupt)) INT_Excep_CMT0_CMI0(void) {
#endif
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_TickCnt++;
#endif
  OS_INT_EnterNestable();
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_Process();
#endif
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize timer for embOS. We assume PLL and core clock were already
  // set by the HardwareSetup() function that was called from start-up.
  //
  CMT0_MSTPCR &= ~(1u << CMT0_MSTPCR_BIT);               // Activate CMT unit
  CMT0_CMSTR0  = 0x00u;                                  // Stop counter
  CMT0_CMCR    = (1u << 6u);                             // Compare match interrupt enable, PCLK/8
  CMT0_CMCOR   = (OS_U16)(OS_TIMER_FREQ / OS_INT_FREQ);  // Count value for 1 msec resolution
  //
  // Enable CMT0 interrupt in interrupt controller
  //
  CMT0_IPR  = OS_TIMER_INT_PRIO;                         // Set interrupt priority, lowest priority
  CMT0_IER |= (1u << CMT0_IER_BIT);                      // Enable CMT interrupt
  IRR_PEND  = 0u;                                        // Clear interrupt request
  ISEL_SRC  = 0u;                                        // Select interrupt source CPU
  //
  // Start timer
  //
  CMT0_CMSTR0  = 0x01u;                                  // Start counter
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Configure and initialize SEGGER SystemView
  //
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_Conf();
#endif
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SetpfOnRx(OS_COM_OnRx);
  JLINKRX_SetpfOnTx(OS_COM_OnTx);
  JLINKRX_SetpfGetNextChar(OS_COM_GetNextChar);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
#if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
  #ifdef __ICCRX__
    __wait_for_interrupt();
  #endif
  #ifdef __RX
    wait();
  #endif
  #ifdef __GNUC__
    __builtin_rx_wait();
  #endif
#endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       HardwareSetup()
*
*  Function description
*    Initialize clock generation and PLL
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before HardwareSetup() is called,
*    no access or call which relies on any variable can be used here
*    and no variables can be set to a specific value as long as those variables
*    are not declared as "no init".
*
*    This function is called from startup code.
*/
void HardwareSetup(void) {
  volatile OS_U32 i;
  volatile OS_U8  v;

  //
  // Initialize PLL, assuming external crystal of 12.0 MHz
  //
                                       // init PLL, multipiers: 0 => x8; 1 => x4; 2 => x2; 3 => x1;
  SCKCR   = 0x00u                      // b7..b0:   Always 0
          | (1uL << 8u)                // b11..b8:  PCK Peripheral module clock multiplier x4 => 48MHz
          | (2uL << 16u)               // b19..b16: BCK External BUS clock multiplier x2 => 24MHz
          | (0uL << 22u)               // b22:      PSTOP0 0 => SDCLK output enabled
          | (0uL << 23u)               // b23:      PSTOP1 0 => BCLK output enabled
          | (0uL << 24u)               // b27..b24: ICK System clock speed multiplier x8 => 96 MHz
          ;
  //
  // Set MSTPCRA, MSTPCRB and MSTPCRC to default setting
  //
  MSTPCRA = 0x47FFFFFFu;               // Set to initial value (Reset condition).
  MSTPCRB = 0xFFFFFFFFu;               // All modules in MSTPCRB stop.
  MSTPCRC = 0xFFFF0000u;               // b0: RAM0, 0 => enabled
                                       // b1: RAM1, 0 => enabled
  //
  // SDRAM Initialization
  //
  SYSCR0        = 0x5A03u;              // Enable the external bus...must also set key and internal ROME bit
  IOPORT_PF4BUS = 0xFFu;                // Enable A0-A15
  IOPORT_PF5BUS = 0x10u;                // Enable D15-D31
  IOPORT_PF6BUS = 0xD0u;                // Enable SDRAM pins
  for (i = 0u ; i < 10000u; i++) {      // Delay at least 100uS after SDCLK active
#ifdef __ICCRX__
    __asm volatile("nop");
#endif
#ifdef __RX
    nop();
#endif
#ifdef __GNUC__
    __asm volatile("nop");
#endif
  }
  BSC_SDIR      = 0x00F0u;              // SDRAM initialization...AutoRefresh interval=3 cycles, Precharge cycles count=3cycles, 15 times
  v             =  BSC_SDICR;
  v            |= (1u << INIRQ);        // Starte SDRAM initialization
  BSC_SDICR     = v;
  do {
    v = (BSC_SDSR & (1u << INIST));
  } while (v == (1u << INIST));         // Wait for initialization complete
  BSC_SDCMOD    = 0u;                   // Set endian to little
  v             = BSC_SDCCR;
  v            &= ~(3u << BSIZE);       // Set SDRAM to 16bit access
  BSC_SDCCR     = v;
  BSC_SDMOD     = 0x0220u;              // Set write single location, CL=2
  BSC_SDRFCR    = (OS_U16)((4u << REFW) | (719u << RFC));
  BSC_SDTR      = 0x000002u;            // Set RAS=1, RCD=1, RP=1, WR=1, CL=2, cycles
  BSC_SDADR     = 0u;                   // Set shift to 8 bits
  BSC_SDRFEN    = 1u;                   // Enable auto refresh
  BSC_SDCCR    |= (1u << EXENB);        // Enable SDRAM operation...this should be last operation
}

#ifdef __ICCRX__
/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  HardwareSetup();
  return 1;  // Always return 1 to enable segment initilization !
}
#endif

/*************************** End of file ****************************/
