/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RSKRX630 Starter-Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

/****** Addresses of sfrs and peripheral identifiers ****************/
#define P2_BASE_ADDR  (0x0008C002u)
#define PC_BASE_ADDR  (0x0008C00Cu)

/****** SFRs and identifiers used for LED demo **********************/
#define P2_DDR        (*(volatile EVENACCESS unsigned char*) (P2_BASE_ADDR + 0x00u))
#define P2_DR         (*(volatile EVENACCESS unsigned char*) (P2_BASE_ADDR + 0x20u))
#define PC_DDR        (*(volatile EVENACCESS unsigned char*) (PC_BASE_ADDR + 0x00u))
#define PC_DR         (*(volatile EVENACCESS unsigned char*) (PC_BASE_ADDR + 0x20u))

/****** Assign LEDs to Ports ****************************************/
#define LED0_BIT      (5u)      // LED 0 on RSKRX630
#define LED1_BIT      (4u)      // LED 1 on RSKRX630
#define LED0_MASK     (1u << LED0_BIT)
#define LED1_MASK     (1u << LED1_BIT)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PC_DDR |= LED0_MASK;  // Switch LED-Port to output
  PC_DR  |= LED0_MASK;  // Switch off leds initially (active low)
  P2_DDR |= LED1_MASK;  // Switch LED-Port to output
  P2_DR  |= LED1_MASK;  // Switch off leds initially (active low)
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PC_DR &= ~LED0_MASK;  // Switch LED on, active low
  } else if (Index == 1) {
    P2_DR &= ~LED1_MASK;  // Switch LED on, active low
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PC_DR |= LED0_MASK;  // Switch LED off, set port high
  } else if (Index == 1) {
    P2_DR |= LED1_MASK;  // Switch LED off, set port high
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {  /* LED is switched off */
    PC_DR ^= LED0_MASK;
  } else if (Index == 1) {
    P2_DR ^= LED1_MASK;
  }
}

/****** End Of File  ************************************************/
