/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Serious Integrated SIM225 evalboard
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

#define P9_BASE_ADDR  (0x0008C009u)
#define P9_DDR        (*(volatile EVENACCESS unsigned char*) (P9_BASE_ADDR + 0x00u))
#define P9_DR         (*(volatile EVENACCESS unsigned char*) (P9_BASE_ADDR + 0x20u))

#define LED0_BIT      (0u)
#define LED1_BIT      (1u)
#define LED2_BIT      (2u)
#define LED3_BIT      (3u)
#define LED_MASK_ALL  ((1u << LED0_BIT) | (1u << LED1_BIT) | (1u << LED2_BIT) | (1u << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  P9_DDR |= LED_MASK_ALL;      // Switch LED-Pins to output 
  P9_DR  &= ~LED_MASK_ALL;     // Switch off LEDs initially (active high)
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 4) {
    P9_DR |= (1u << Index);    // Switch LED off
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 4) {
    P9_DR &= ~(1u << Index);   // Switch LED on
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < 4) {
    P9_DR ^= (1u << Index);
  }
}

/****** End Of File *************************************************/
