/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for YRDKRX63N Starter-Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PD_BASE_ADDR        (0x0008C00Du)
#define PE_BASE_ADDR        (0x0008C00Eu)

/****** SFRs and identifiers used for LED demo **********************/
#define PD_DDR              (*(volatile EVENACCESS unsigned char*) (PD_BASE_ADDR + 0x00u))
#define PD_DR               (*(volatile EVENACCESS unsigned char*) (PD_BASE_ADDR + 0x20u))
#define PE_DDR              (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x00u))
#define PE_DR               (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x20u))

/****** Assign LEDs to Ports ****************************************/
#define LED_PORT1_DATA      PD_DR
#define LED_PORT1_DIRECTION PD_DDR
#define LED_PORT2_DATA      PE_DR
#define LED_PORT2_DIRECTION PE_DDR

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LED_PORT1_DIRECTION  = 0xFFu;     // Switch LED-Port to output
  LED_PORT1_DATA       = 0xFFu;     // Switch off LEDs initially (active low)
  LED_PORT2_DIRECTION |= 0x0Fu;     // Switch LED-Port to output
  LED_PORT2_DATA      |= 0x0Fu;     // Switch off LEDs initially (active low)
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 8) {
    LED_PORT1_DATA &= ~(1u << Index);   // Switch LED on
  } else {
    Index -= 8;
    if (Index < 4) {
      LED_PORT2_DATA &= ~(1u << Index); // Switch LED on
    }
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 8) {
    LED_PORT1_DATA |= (1u << Index);    // Switch LED off
  } else {
    Index -= 8;
    if (Index < 4) {
      LED_PORT2_DATA |= (1u << Index);  // Switch LED off
    }
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < 8) {
    LED_PORT1_DATA ^= (1u << Index);
  } else {
    Index -= 8;
    if (Index < 4) {
      LED_PORT2_DATA ^= (1u << Index);
    }
  }
}

/****** End Of File  ************************************************/
