/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : BSP for RSKRX66T Starter kit
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

#define LED0               (1u << 5u)  // P95 (Pin45) green
#define LED1               (1u << 4u)  // P94 (Pin46) orange
#define LED2               (1u << 3u)  // P93 (Pin47) red
#define LED3               (1u << 0u)  // PE0 (Pin17) red

#define P9_BASE_ADDR       (0x0008C009u)
#define P9_PDR             (*(volatile EVENACCESS unsigned char*) (P9_BASE_ADDR + 0x00u))
#define P9_PODR            (*(volatile EVENACCESS unsigned char*) (P9_BASE_ADDR + 0x20u))

#define PE_BASE_ADDR       (0x0008C00Eu)
#define PE_PDR             (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x00u))
#define PE_PODR            (*(volatile EVENACCESS unsigned char*) (PE_BASE_ADDR + 0x20u))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Switch off LEDs
  //
  P9_PODR |= LED0 | LED1 | LED2;
  PE_PODR |= LED3;
  //
  // Set I/O ports to output
  //
  P9_PDR |= LED0 | LED1 | LED2;
  PE_PDR |= LED3;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch (Index) {
    case 0:
      P9_PODR &= ~LED0;
      break;
    case 1:
      P9_PODR &= ~LED1;
      break;
    case 2:
      P9_PODR &= ~LED2;
      break;
    case 3:
      PE_PODR &= ~LED3;
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch (Index) {
    case 0:
      P9_PODR |= LED0;
      break;
    case 1:
      P9_PODR |= LED1;
      break;
    case 2:
      P9_PODR |= LED2;
      break;
    case 3:
      PE_PODR |= LED3;
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  switch (Index) {
    case 0:
      P9_PODR ^= LED0;
      break;
    case 1:
      P9_PODR ^= LED1;
      break;
    case 2:
      P9_PODR ^= LED2;
      break;
    case 3:
      PE_PODR ^= LED3;
      break;
    default:
      break;
  }
}

/*************************** End of file ****************************/
