/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_RX71M.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (240000000u / 4u / 8u)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKRX.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
//
// Protect register
//
#define PRCR                 (*(volatile EVENACCESS OS_U16*)(0x000803FEu))

//
// SYSTEM module
//
#define SYSTEM_BASE_ADDR     (0x00080000u)
#define SYSCR0               (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x006u))
#define MSTPCRA              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x010u))
#define MSTPCRB              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x014u))
#define MSTPCRC              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x018u))
#define MSTPCRD              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x01Cu))

#define SCKCR                (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x020u))
#define SCKCR2               (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x024u))
#define SCKCR3               (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x026u))
#define PLLCR                (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x028u))
#define PLLCR2               (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x02Au))
#define MOSCCR               (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x032u))
#define SOSCCR               (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x033u))
#define MOSCWTCR             (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x0A2u))
#define MOFCR                (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0xC293u))
#define OSCOVFSR             (*(volatile EVENACCESS OS_U8*) (SYSTEM_BASE_ADDR + 0x003Cu))
#define MEMWAIT              (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x6610u))

#define OSCOVFSR_MOOVF_MASK  (1u << 0)
#define OSCOVFSR_PLOVF_MASK  (1u << 2)

//
// MPC used for SDRAM init
//
#define MPC_BASE_ADDR        (0x0008C100u)
#define PFAOE0               (*(volatile EVENACCESS OS_U8*) (MPC_BASE_ADDR + 0x04u))
#define PFAOE1               (*(volatile EVENACCESS OS_U8*) (MPC_BASE_ADDR + 0x05u))
#define PFBCR0               (*(volatile EVENACCESS OS_U8*) (MPC_BASE_ADDR + 0x06u))
#define PFBCR1               (*(volatile EVENACCESS OS_U8*) (MPC_BASE_ADDR + 0x07u))

//
// BSC for SDRAM init
//
#define BSC_BASE_ADDR        (0x00081300u)
#define SDAMOD               (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02902u))
#define SDIR                 (*(volatile EVENACCESS OS_U16*)(BSC_BASE_ADDR + 0x02924u))
#define SDICR                (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02920u))
#define SDSR                 (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02950u))
#define SDCMOD               (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02901u))
#define SDCCR                (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02900u))
#define SDMOD                (*(volatile EVENACCESS OS_U16*)(BSC_BASE_ADDR + 0x02948u))
#define SDRFCR               (*(volatile EVENACCESS OS_U16*)(BSC_BASE_ADDR + 0x02914u))
#define SDTR                 (*(volatile EVENACCESS OS_U32*)(BSC_BASE_ADDR + 0x02944u))
#define SDADR                (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02940u))
#define SDRFEN               (*(volatile EVENACCESS OS_U8*) (BSC_BASE_ADDR + 0x02916u))

#define PORTA_DSCR           (*(volatile EVENACCESS OS_U8*) (0x00008C0EAu))
#define PORTB_DSCR           (*(volatile EVENACCESS OS_U8*) (0x00008C0EBu))
#define PORTD_DSCR           (*(volatile EVENACCESS OS_U8*) (0x00008C0EDu))
#define PORTE_DSCR           (*(volatile EVENACCESS OS_U8*) (0x00008C0EEu))

//
// Assign timer used for OS
//
#define CMT0_BASE_ADDR       (0x00088000u)
#define CMT0_MSTPCR          (MSTPCRA)
#define CMT0_MSTPCR_BIT      (15u)
#define CMT0_VECT            28u
#define CMT0_IPR             (IPR04)
#define CMT0_IER             (IER03)
#define CMT0_IER_BIT         (4u)
#define OS_TIMER_INT_PRIO    (0x01u)     // Lowest priority
#define OS_TIMER_VECT        CMT0_VECT
#define CMT0_CMSTR0          (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x00u))
#define CMT0_CMCR            (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x02u))
#define CMT0_CMCNT           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x04u))
#define CMT0_CMCOR           (*(volatile EVENACCESS OS_U16*)(CMT0_BASE_ADDR + 0x06u))

//
// Interrupt controller
//
#define IRR_BASE_ADDR        (0x00087000u)
#define ISEL_BASE_ADDR       (0x00087100u)

#define IER03                (*(volatile EVENACCESS OS_U8*) (0x00087203u))
#define IPR04                (*(volatile EVENACCESS OS_U8*) (0x00087304u))

#define IRR_PEND             (*(volatile EVENACCESS OS_U8*) (IRR_BASE_ADDR + OS_TIMER_VECT))
#define ISEL_SRC             (*(volatile EVENACCESS OS_U8*) (ISEL_BASE_ADDR + OS_TIMER_VECT))

/*********************************************************************
*
*       Const data
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration of Option Bytes and ID Code
*
*       We set all ID Code bytes explicitly to 0xFF which is the default value.
*       IAR handles these bytes in IAR linker files.
*       RX GNU handles these bytes in vects.c and GNU linker script.
*/
#ifdef __RX
  #pragma address _MDEreg=0x00120064            // MDE register (Single Chip Mode)
  #ifdef __BIG
    const unsigned long _MDEreg = 0xFFFFFFF8;   // Big endian
  #else
    const unsigned long _MDEreg = 0xFFFFFFFF;   // Little endian
  #endif
  #pragma address _OFS1=0x00120068              // Option function select register 1 (OFS1)
  const unsigned long _OFS1   = 0xFFFFFFFF;
  #pragma address _OFS2=0x0012006C              // Option function select register 2 (OFS2)
  const unsigned long _OFS2   = 0xFFFFFFFF;
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return CMT0_CMCNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return IRR_PEND;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#ifdef __ICCRX__
  #pragma vector = (OS_TIMER_VECT)
  static __interrupt void SysTick_Handler(void) {
#endif
#ifdef __RX
  #pragma interrupt (SysTick_Handler(vect=OS_TIMER_VECT))
  static void SysTick_Handler(void) {
#endif
#ifdef __GNUC__
  void __attribute__ ((interrupt)) INT_Excep_CMT0_CMI0(void) {
#endif
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_TickCnt++;
#endif
  OS_INT_EnterNestable();
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_Process();
#endif
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  PRCR         = 0xA50Bu;                                // Protect off
  CMT0_MSTPCR &= ~(1u << CMT0_MSTPCR_BIT);               // Activate CMT unit
  CMT0_CMSTR0  = 0x00u;                                  // Stop counter
  CMT0_CMCR    = (1u << 6u);                             // Compare match interrupt enable, PCLK/8
  CMT0_CMCOR   = (OS_U16)(OS_TIMER_FREQ / OS_INT_FREQ);  // Count value for 1 msec resolution
  //
  // Enable CMT0 interrupt in interrupt controller
  //
  CMT0_IPR  = OS_TIMER_INT_PRIO;                         // Set interrupt priority, lowest priority
  CMT0_IER |= (1u << CMT0_IER_BIT);                      // Enable CMT interrupt
  IRR_PEND  = 0u;                                        // Clear interrupt request
  ISEL_SRC  = 0u;                                        // Select interrupt source CPU
  //
  // Start timer
  //
  CMT0_CMSTR0  = 0x01u;                                  // Start counter
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Configure and initialize SEGGER SystemView
  //
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_Conf();
#endif
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SetpfOnRx(OS_COM_OnRx);
  JLINKRX_SetpfOnTx(OS_COM_OnTx);
  JLINKRX_SetpfGetNextChar(OS_COM_GetNextChar);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  PRCR = 0xA500u;                                        // Protect on
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
#if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
  #ifdef __ICCRX__
    __wait_for_interrupt();
  #endif
  #ifdef __RX
    wait();
  #endif
  #ifdef __GNUC__
    __builtin_rx_wait();
  #endif
#endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       HardwareSetup()
*
*  Function description
*    Initialize clock generation and PLL
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before HardwareSetup() is called,
*    no access or call which relies on any variable can be used here
*    and no variables can be set to a specific value as long as those variables
*    are not declared as "no init".
*
*    This function is called from startup code.
*/
void HardwareSetup(void) {
  //
  // Initialize PLL, assuming external crystal of 24.0 MHz
  //
  //  Clock Description              Frequency
  //  ----------------------------------------
  //  Input Clock Frequency............  24 MHz
  //  PLL frequency (x10).............. 240 MHz
  //  Internal Clock Frequency......... 120 MHz
  //  Flash Clock Frequency............  60 MHz
  //  Peripheral Clock Frequency.......  60 MHz
  //  USB Clock Frequency..............  48 MHz
  //  External Bus Clock Frequency.....  60 MHz
  //
  volatile OS_U32 Memorywaitcycle;

  PRCR     = 0xA50Bu;             // Protect off
  //
  // Set main clock control registers
  //
  MOFCR    = 0x00u;
  MOSCWTCR = 0x5Cu;               // Set wait time
  MOSCCR   = 0x00u;               // EXTAL ON
  while ((OSCOVFSR & OSCOVFSR_MOOVF_MASK) != OSCOVFSR_MOOVF_MASK) {  // Wait for main clock oscillator wait counter overflow
  }
  //
  // Set PLL circuit
  //
  PLLCR  = 0x1300u;               // PLL x10
  PLLCR2 = 0x00u;                 // PLL ON
  while ((OSCOVFSR & OSCOVFSR_PLOVF_MASK) != OSCOVFSR_PLOVF_MASK) {  // Wait for PLL wait counter overflow
  }
  // SCKCR - System Clock Control Register
  //  FCK[3:0]    0x02 = Flash clock: PLL = (240 / 4) = 60 MHz
  //  ICK[3:0]    0x00 = System clock: PLL = (240 / 1) = 240 MHz
  //  PSTOP1      0x00 = BCLK pin output is enabled
  //  BCK[3:0]    0x02 = BCLK: PLL/4 = 60 MHz
  //  PCKA[3:0]   0x02 = Peripheral clock A: PLL/4 = 60 MHz
  //  PCKB[3:0]   0x02 = Peripheral clock B: PLL/4 = 60 MHz
  //  PCKC[3:0]   0x02 = Peripheral clock C: PLL/4 = 60 MHz
  //  PCKD[3:0]   0x02 = Peripheral clock D: PLL/4 = 60 MHz
  //
  SCKCR           = 0x20022222u;  // FCK=PLL/4,ICK=PLL/1,BCK=PLL/4,PCK=PLL/4
  SCKCR2          = 0x0041;       // Set UCLK for USB
  MEMWAIT         = 1u;           // Set memory wait cycle setting register
  Memorywaitcycle = MEMWAIT;
  SCKCR3          = 0x0400;       // Set PLL as clock source
  PRCR            = 0xA500u;      // Protect on
  //
  // Set MSTPCRA, MSTPCRB and MSTPCRC to default setting
  //
  MSTPCRA = 0x47FFFFFFu;          // Set to initial value (Reset condition).
  MSTPCRB = 0xFFFFFFFFu;          // All modules in MSTPCRB stop.
  MSTPCRC = 0xFFFF0000u;          // b0: RAM0, 0 => enabled
                                  // b1: RAM1, 0 => enabled
  MSTPCRD = 0xFFFFFF00u;          // Set to initial value (Reset condition).
  //
  // SDRAM Initialization
  //
  PFAOE0     = 0xff;              // Enable SDRAM pins
  PFAOE1     = 0x1f;
  PFBCR0     = 0x51;
  PFBCR1     = 0xda;
  PORTA_DSCR = 0xff;
  PORTB_DSCR = 0xff;
  PORTD_DSCR = 0xff;
  PORTE_DSCR = 0xff;

  PRCR   = 0xA50Bu;               // Protect off
  SYSCR0 = 0x5A03;                // Must also set key and internal ROME bit (5A = Key, 0003 = Set ROME and EXBE bit).
  SDIR   = 0x0020;                // Precharge cycles count
  while(SDSR != 0x00) {
  }
  SDICR = 0x1;                    // Start SDRAM init.
  while(SDSR != 0x00) {           // Wait for init complete.
  }
  SDMOD  = 0x0230;                // Set write single location
  SDTR   = 0x00021203;
  SDADR  = 1;                     // Set shift to 9 bits (for 8Mx16 SDRAM).
  SDCMOD = 0;                     // Set to little-endian.
  SDAMOD = 0;
  SDRFCR = 0x53a8;
  SDRFEN = 1;                     // En auto-refresh.
  SDCCR  = 1;                     // En SDRAM operation. Should be last operation.
  PRCR   = 0xA500u;               // Protect on
}

#ifdef __ICCRX__
/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  HardwareSetup();
  return 1;  // Always return 1 to enable segment initilization !
}
#endif

/*************************** End of file ****************************/
