/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_RX72N.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (240000000u / 4u / 8u)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKRX.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
//
// SYSTEM module
//
#define SYSTEM_BASE_ADDR             (0x00080000u)
#define SYSTEM_MSTPCRA               (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x0010u))  // Module stop control register
#define SYSTEM_SCKCR                 (*(volatile EVENACCESS OS_U32*)(SYSTEM_BASE_ADDR + 0x0020u))  // System clock control register
#define SYSTEM_SCKCR2                (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x0024u))  // System clock control register 2
#define SYSTEM_SCKCR3                (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x0026u))  // System clock control register 3
#define SYSTEM_PLLCR                 (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x0028u))  // PLL control register
#define SYSTEM_PLLCR2                (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x002Au))  // PLL control register 2
#define SYSTEM_PACKCR                (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x0044u))  // Specific-Use Clock Control Register
#define SYSTEM_PPLLCR                (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x0048u))  // PPLL control register
#define SYSTEM_PPLLCR2               (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x004Au))  // PPLL control register 2
#define SYSTEM_PPLLCR3               (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x004Bu))  // PPLL control register 3

#define SYSTEM_MOSCCR                (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x0032u))  // Main clock oscillator control register
#define SYSTEM_OSCOVFSR              (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x003Cu))  // Oscillating stabilization flag register
#define SYSTEM_MOSCWTCR              (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x00A2u))  // Main clock oscillator wait control register
#define SYSTEM_PRCR                  (*(volatile EVENACCESS OS_U16*)(SYSTEM_BASE_ADDR + 0x03FEu))  // Register write protection function
#define SYSTEM_MEMWAIT               (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0x101Cu))  // Memory wait cycle setting register
#define SYSTEM_MOFCR                 (*(volatile EVENACCESS OS_U8* )(SYSTEM_BASE_ADDR + 0xC293u))  // Main clock oscillator forced oscillation control register

#define SYSTEM_MSTPCRA_MSTPA15       (1u << 15)  // Target module: CMT unit 0 (CMT0, CMT1)
#define SYSTEM_OSCOVFSR_MOOVF        (1u <<  0)
#define SYSTEM_OSCOVFSR_PLOVF        (1u <<  2)
#define SYSTEM_OSCOVFSR_PPLOVF       (1u <<  5)
#define SYSTEM_PRCR_PRC0             (1u <<  0)
#define SYSTEM_PRCR_PRC1             (1u <<  1)
#define SYSTEM_PRCR_PRKEY            (0xA500u)

//
// Interrupt control unit
//
#define ICU_BASE_ADDR                (0x00087000u)
#define ICU_IR028                    (*(volatile EVENACCESS OS_U8*)(ICU_BASE_ADDR + 0x01Cu))       // Interrupt request register 28
#define ICU_IER03                    (*(volatile EVENACCESS OS_U8*)(ICU_BASE_ADDR + 0x203u))       // Interrupt request enable register 3
#define ICU_IPR004                   (*(volatile EVENACCESS OS_U8*)(ICU_BASE_ADDR + 0x304u))       // Interrupt source priority register 4

//
// Assign timer used for OS
//
#define CMT_BASE_ADDR                (0x00088000u)
#define CMT_CMSTR0                   (*(volatile EVENACCESS OS_U16*)(CMT_BASE_ADDR + 0x00u))       // Compare match timer start register 0
#define CMT0_CMCR                    (*(volatile EVENACCESS OS_U16*)(CMT_BASE_ADDR + 0x02u))       // CMT0 Compare match timer control register
#define CMT0_CMCNT                   (*(volatile EVENACCESS OS_U16*)(CMT_BASE_ADDR + 0x04u))       // CMT0 Compare match counter register
#define CMT0_CMCOR                   (*(volatile EVENACCESS OS_U16*)(CMT_BASE_ADDR + 0x06u))       // CMT0 Compare match constant register

#define CMT0_VECT                    28u
#define CMT0_IR                      (ICU_IR028)
#define CMT0_IPR                     (ICU_IPR004)
#define CMT0_IER                     (ICU_IER03)
#define CMT0_IER_IEN                 (1u << 4)
#define CMT0_INT_PRIO                (0x01u)  // Lowest priority

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return CMT0_CMCNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return CMT0_IR;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#ifdef __ICCRX__
  #pragma vector = (CMT0_VECT)
  static __interrupt void SysTick_Handler (void) {
#endif
#ifdef __RX
  #pragma interrupt (SysTick_Handler(vect=CMT0_VECT))
  static void SysTick_Handler (void) {
#endif
#ifdef __GNUC__
  void __attribute__ ((interrupt)) SysTick_Handler (void) {
#endif
  OS_INT_EnterNestable();
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_Process();
#endif
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  SYSTEM_PRCR     = SYSTEM_PRCR_PRKEY | SYSTEM_PRCR_PRC1;  // Disable register protection
  SYSTEM_MSTPCRA &= ~(SYSTEM_MSTPCRA_MSTPA15);             // Activate CMT unit 0
  SYSTEM_PRCR     = SYSTEM_PRCR_PRKEY;                     // Enable register protection
  CMT_CMSTR0     &= ~1u;                                   // Stop counter
  CMT0_CMCR       = (0u << 0)                              // Clock select: PCLKB/8
                  | (1u << 6);                             // Enable compare match interrupt
  CMT0_CMCOR      = (OS_U16)OS_TIMER_RELOAD;               // Count value for 1 msec resolution
  //
  // Enable timer interrupt in interrupt controller
  //
  CMT0_IPR     = CMT0_INT_PRIO;                            // Set interrupt priority, lowest priority
  CMT0_IER    |= CMT0_IER_IEN;                             // Enable CMT interrupt
  CMT0_IR      = 0u;                                       // Clear interrupt request
  CMT_CMSTR0  |= 1u;                                       // Start counter
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SetpfOnRx(OS_COM_OnRx);
  JLINKRX_SetpfOnTx(OS_COM_OnTx);
  JLINKRX_SetpfGetNextChar(OS_COM_GetNextChar);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
#if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
  #ifdef __ICCRX__
    __wait_for_interrupt();
  #endif
  #ifdef __RX
    wait();
  #endif
  #ifdef __GNUC__
    __builtin_rx_wait();
  #endif
#endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKRX_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       HardwareSetup()
*
*  Function description
*    Initialize clock generation and PLL
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before HardwareSetup() is called,
*    no access or call which relies on any variable can be used here
*    and no variables can be set to a specific value as long as those variables
*    are not declared as "no init".
*
*    This function is called from startup code.
*/
void HardwareSetup(void) {
  //
  // Main clock (XTAL):      16 MHz (    8-24 MHz)
  // PLL output:            240 MHZ
  // PPLL output:           200 MHz
  // System clock (ICLK):   240 MHz (max: 240 MHz)
  // Flash-IF clock (FCLK):  60 MHz (max:  60 MHz)
  // PCLKA:                 120 MHz (max: 120 MHz)
  // PCLKB:                  60 MHz (max:  60 MHz)
  // PCLKC:                  60 MHz (max:  60 MHz)
  // PCLKD:                  60 MHz (max:  60 MHz)
  // BCLK:                   80 MHz (max: 120 MHz)
  // BCLK pin:               80 MHz (max:  80 MHz)
  // SDCLK pin:              80 MHz (max:  80 MHz)
  // UDIVCLK:                48 MHz
  // USB clock (UCLK):       48 MHZ (max:  48 MHz)
  // Ethernet-PHY:           --
  //
  SYSTEM_PRCR = SYSTEM_PRCR_PRKEY
              | SYSTEM_PRCR_PRC0
              | SYSTEM_PRCR_PRC1;                          // Disable register protection
  //
  // Initialize main clock oscillator
  //
  SYSTEM_MOFCR    = (2u << 4);                             // MODRV2: Set driving ability 8.1-16 MHz
  SYSTEM_MOSCWTCR = 0x53u;                                 // MSTS: Set oscillating settling time for main clock oscillator to 10ms
  SYSTEM_MOSCCR   = 0u;                                    // Start main clock oscillator
  while ((SYSTEM_OSCOVFSR & SYSTEM_OSCOVFSR_MOOVF) == 0);  // Wait for main clock to become stable
  //
  // Initialize PLL
  //
  SYSTEM_PLLCR  = (0x1Du << 8);                            // Set frequency multiplication factor to *15
  SYSTEM_PLLCR2 = 0u;                                      // Enable PLL
  while ((SYSTEM_OSCOVFSR & SYSTEM_OSCOVFSR_PLOVF) == 0);  // Wait for PLL to become stable
  //
  // Switch clock sources to PLL and adjust dividers for each derived clock
  //
  SYSTEM_MEMWAIT = 1u;                                     // Adjust wait states for on-chip SRAM (needed with ICLK > 120 MHz)
  SYSTEM_SCKCR   = (2u <<  0)                              // PCKD: PCLKD = PLL / 4
                 | (2u <<  4)                              // PCKC: PCLKC = PLL / 4
                 | (2u <<  8)                              // PCKB: PCLKB = PLL / 4
                 | (1u << 12)                              // PCKC: PCLKA = PLL / 2
                 | (9u << 16)                              // BCK:  BCLK  = PLL / 3
                 | (0u << 24)                              // ICK:  ICLK  = PLL / 1
                 | (2u << 28);                             // FCK:  FCLK  = PLL / 4
  SYSTEM_SCKCR3 |= (4u <<  8);                             // Switch to PLL
  SYSTEM_PPLLCR  = (0u   << 0)                             // Inp(Clk) / 1
                 | (0x18 << 8);                            // PPLL multiplier = 12.5
  SYSTEM_PPLLCR2 = 0;                                      // Start the PPLL operation
  while ((SYSTEM_OSCOVFSR & SYSTEM_OSCOVFSR_PPLOVF) == 0); // Wait for main clock to become stable
  SYSTEM_PPLLCR3 = 0;                                      // PPLL Output / 2
  SYSTEM_PACKCR |= (1u << 4);                              // CLKOUT25M = PPLL -> for ENET PHY
  SYSTEM_SCKCR2  = (1u << 0)                               // Reserved, write should be 1
                 | (4u << 4);                              // UCK:  UCLK  = InputPLL / 5
  SYSTEM_PRCR    = SYSTEM_PRCR_PRKEY;                      // Enable register protection
}

#ifdef __ICCRX__
/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  HardwareSetup();
  return 1;  // Always return 1 to enable segment initilization !
}
#endif

/*************************** End of file ****************************/
