/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: BSP for Renesas RSKRX72T eval board
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifdef __RX
  #define EVENACCESS  __evenaccess
#else
  #define EVENACCESS
#endif

#define LED0               (1u << 5u)  // P112 (Pin54) green
#define LED1               (1u << 4u)  // P111 (Pin55) orange
#define LED2               (1u << 3u)  // P110 (Pin60) red
#define LED3               (1u << 0u)  // P109 (Pin61) red


#define P5_BASE_ADDR       (0x0008C005u)
#define P5_PDR             (*(volatile EVENACCESS unsigned char*) (P5_BASE_ADDR + 0x00u))
#define P5_PODR            (*(volatile EVENACCESS unsigned char*) (P5_BASE_ADDR + 0x20u))

#define P6_BASE_ADDR       (0x0008C006u)
#define P6_PDR             (*(volatile EVENACCESS unsigned char*) (P6_BASE_ADDR + 0x00u))
#define P6_PODR            (*(volatile EVENACCESS unsigned char*) (P6_BASE_ADDR + 0x20u))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Switch off LEDs
  //
  P5_PODR |= LED0 | LED1;
  P6_PODR |= LED2 | LED3;
  //
  // Set I/O ports to output
  //
  P5_PDR |= LED0 | LED1;
  P6_PDR |= LED2 | LED3;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch (Index) {
    case 0:
      P5_PODR &= ~LED0;
      break;
    case 1:
      P5_PODR &= ~LED1;
      break;
    case 2:
      P6_PODR &= ~LED2;
      break;
    case 3:
      P6_PODR &= ~LED3;
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch (Index) {
    case 0:
      P5_PODR |= LED0;
      break;
    case 1:
      P5_PODR |= LED1;
      break;
    case 2:
      P6_PODR |= LED2;
      break;
    case 3:
      P6_PODR |= LED3;
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  switch (Index) {
    case 0:
      P5_PODR ^= LED0;
      break;
    case 1:
      P5_PODR ^= LED1;
      break;
    case 2:
      P6_PODR ^= LED2;
      break;
    case 3:
      P6_PODR ^= LED3;
      break;
    default:
      break;
  }
}

/*************************** End of file ****************************/
